/* copyright    Maher Awamy <muhri@muhri.net>           *
*    released under the GPL                              */
/* Thanks to Epiphany Code for the messages and making this a breeze for me! */
#include <sys/time.h>
#include "SkipStoneBadCertListener.h"
#include <nsStringAPI.h>
#include <nsIX509Cert.h>
#include <nsIX509CertValidity.h>
#include <nsCOMPtr.h>
#include "intl.h"
#include <gtk/gtk.h>


NS_IMPL_ISUPPORTS1(SkipStoneBadCertListener, nsIBadCertListener)

static gint create_cert_dialog(gchar *title, gchar *text, gchar *checkbox_text, gboolean *checkbox_value)
{
   

   GtkWidget *dialog, *label;
   GtkWidget *checkbox = NULL;
   gint response; 
   dialog = gtk_dialog_new();
   gtk_window_set_title(GTK_WINDOW(dialog), title);
   label = gtk_label_new(NULL);
   gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
   gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
   gtk_label_set_selectable(GTK_LABEL(label), TRUE);
   gtk_label_set_markup(GTK_LABEL(label), text);
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
   gtk_dialog_add_buttons(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			    GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
   if (checkbox_text) 
     {
	checkbox = gtk_check_button_new_with_label(checkbox_text);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), *checkbox_value);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), checkbox, TRUE, TRUE, 0);
     } 
   
   gtk_widget_show_all(dialog);
   response = gtk_dialog_run(GTK_DIALOG(dialog));
   if (response == GTK_RESPONSE_ACCEPT && checkbox) 
     {
	*checkbox_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbox));
     }
   
   gtk_widget_destroy(dialog);
   return response;
   
   
   
}
 
  
  
SkipStoneBadCertListener::SkipStoneBadCertListener()
{
  /* member initializers and constructor code */
}

SkipStoneBadCertListener::~SkipStoneBadCertListener()
{
  /* destructor code */
}

/* boolean confirmUnknownIssuer (in nsIInterfaceRequestor socketInfo, in nsIX509Cert cert, out short certAddType); */
NS_IMETHODIMP SkipStoneBadCertListener::ConfirmUnknownIssuer(nsIInterfaceRequestor *socketInfo, nsIX509Cert *cert, PRInt16 *certAddType, PRBool *_retval)
{
   gboolean accept_perm;
   gchar *secondary, *tertiary, *message;
   gint response;
     
   nsString commonName;
   cert->GetCommonName (commonName);
   
   NS_ConvertUTF16toUTF8 cCommonName (commonName);
   
   secondary = g_markup_printf_escaped (_("It was not possible to automatically trust “%s”. "
					  "It is possible that someone is intercepting your "
					  "communication to obtain your confidential information."),
					cCommonName.get());
   
   tertiary = g_markup_printf_escaped (_("You should only connect to the site if you are certain "
					 "you are connected to “%s”."),
				       cCommonName.get());
   
   message = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
			      _("Connect to untrusted site?"),
			      secondary, tertiary);
   response = create_cert_dialog(_("Untrusted Site"), message, _("Trust this security information from now on"), &accept_perm);
   if (response != GTK_RESPONSE_ACCEPT) 
     {
	*_retval = PR_FALSE;
	*certAddType = UNINIT_ADD_FLAG;
     }
    else
     {
	if (accept_perm) 
	  {
	     *_retval = PR_TRUE;
	     *certAddType = ADD_TRUSTED_PERMANENTLY;
	     
	  }
	
	else
	  {
	     
	     *_retval = PR_TRUE;
	     *certAddType = ADD_TRUSTED_FOR_SESSION;
	  }
	
     }
   
	
   return NS_OK;
}

/* boolean confirmMismatchDomain (in nsIInterfaceRequestor socketInfo, in AUTF8String targetURL, in nsIX509Cert cert); */
NS_IMETHODIMP SkipStoneBadCertListener::ConfirmMismatchDomain(nsIInterfaceRequestor *socketInfo, const nsACString & targetURL, nsIX509Cert *cert, PRBool *_retval)
{
   gint response;
   gchar *message, *first, *second;
   nsString commonName;
   cert->GetCommonName(commonName);
   NS_ConvertUTF16toUTF8 cCommonName (commonName);
   nsCString cTargetUrl (targetURL);
   first = g_markup_printf_escaped (_("The site “%s” returned security information for "
				      "“%s”. It is possible that someone is intercepting "
				      "your communication to obtain your confidential "
				      "information."),
				    cTargetUrl.get(), cCommonName.get());
   
   second = g_markup_printf_escaped (_("You should only accept the security information if you "
				       "trust “%s” and “%s”."),
				     cTargetUrl.get(), cCommonName.get());
   
   message = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
			  _("Accept incorrect security information?"),
			  first, second);
   
   response = create_cert_dialog(_("Domain Mismatch"), message, NULL, NULL);
   g_free(second);
   g_free(first);
   g_free(message);
   *_retval = (response == GTK_RESPONSE_ACCEPT);
   return NS_OK;

}

/* boolean confirmCertExpired (in nsIInterfaceRequestor socketInfo, in nsIX509Cert cert); */
NS_IMETHODIMP SkipStoneBadCertListener::ConfirmCertExpired(nsIInterfaceRequestor *socketInfo, nsIX509Cert *cert, PRBool *_retval)
{
   nsresult rv;
   PRTime now = PR_Now();
   PRTime notAfter, notBefore, timeToUse;
   PRInt64 normalizedTime;
   time_t t;
   struct tm tm;
   gchar formattedDate[128];
   gchar *fdate;
   const gchar *primary, *text;
   gchar *secondary, *message;
   
   *_retval = PR_FALSE;
   
   nsCOMPtr<nsIX509CertValidity> validity;
   rv = cert->GetValidity (getter_AddRefs(validity));
   if (NS_FAILED(rv)) return rv;
   rv = validity->GetNotAfter (&notAfter);
   if (NS_FAILED(rv)) return rv;
   
   rv = validity->GetNotBefore (&notBefore);
   if (NS_FAILED(rv)) return rv;
   
   if (LL_CMP(now, >, notAfter))
     {
	
	primary = _("Accept expired security information?");
	/* Translators: first %s is a hostname, second %s is a time/date */
	text    = _("The security information for “%s” "
		    "expired on %s.");
	timeToUse = notAfter;
     }
   
   else
     {
	
	primary = _("Accept not yet valid security information?");
	/* Translators: first %s is a hostname, second %s is a time/date */
	text    = _("The security information for “%s” isn't valid until %s.");
	timeToUse = notBefore;
     }
   
   nsString commonName;
   cert->GetCommonName (commonName);
   
   NS_ConvertUTF16toUTF8 cCommonName (commonName);
   
   LL_DIV (normalizedTime, timeToUse, PR_USEC_PER_SEC);
   LL_L2UI (t, normalizedTime);
   /* To translators: this a time format that is used while displaying the
    *          * expiry or start date of an SSL certificate, for the format see 
    *          * strftime(3) */
   strftime (formattedDate, sizeof(formattedDate), _("%a %d %b %Y"),
	     localtime_r (&t, &tm));
   /* FIXME! this isn't actually correct, LC_CTIME codeset could be different than locale codeset! */
   fdate = g_locale_to_utf8 (formattedDate, -1, NULL, NULL, NULL);
   
   secondary = g_markup_printf_escaped (text, cCommonName.get(), fdate);
   
   message = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
			      primary, secondary,
			      _("You should ensure that your computer's time is correct."));
   
   gint res = create_cert_dialog(_("Expired Security Information"), message, NULL, NULL);
   
   g_free (fdate);
   g_free (message);
   g_free (secondary);
   
   *_retval = (res == GTK_RESPONSE_ACCEPT);
   
   return NS_OK;
   
}

/* void notifyCrlNextupdate (in nsIInterfaceRequestor socketInfo, in AUTF8String targetURL, in nsIX509Cert cert); */
NS_IMETHODIMP SkipStoneBadCertListener::NotifyCrlNextupdate(nsIInterfaceRequestor *socketInfo, const nsACString & targetURL, nsIX509Cert *cert)
{
   nsCString cTargetUrl (targetURL);
   
   nsString commonName;
   cert->GetCommonName (commonName);
   
   GtkWidget *dialog = gtk_message_dialog_new (NULL,
					       GTK_DIALOG_DESTROY_WITH_PARENT,
					       GTK_MESSAGE_ERROR,
					       GTK_BUTTONS_OK,
					       _("Cannot establish connection to “%s”"),
					       cTargetUrl.get ());
   
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
					     _("The certificate revocation list (CRL) from “%s” "
					       "needs to be updated.\n\n"
					       "Please ask your system administrator for assistance."),
					     NS_ConvertUTF16toUTF8 (commonName).get ());
      
   g_signal_connect (dialog, "response",(GCallback) gtk_widget_destroy, NULL);
   gtk_widget_show_all (dialog);
   return NS_OK;
   
}

